(if (not (and dxf ustr))                                ;test subroutines   
  (prompt "\nRequires DXF and USTR functions. Load aborted. ")       
  (progn                                                ;else load OK

;* AUTOBLK.LSP contains routines that allow the placement of a BLOCK on a
;* PLINE or LINE and automatically break the line out of the Block area.
;* The block must be built with its first entity a constant invisible attribute
;* tagged BREAKDIM, containing a value equal to the length of break required.
;* The insertion point must be in the center of the horizontal BREAKDIM location.
;* The DXF, NO-PATH and USTR functions, and the global #DWGSC scale factor are
;* required.

;* GETBK prompts for block name, searches & returns block name & breakdim data.
;* If BLNAME argument is non-nil it serves as the prompt default.
;* It must be supplied a valid BKDIST argument if supplied a name as BLNAME.
(defun getbk (blname bkdist / ok tbdata bldata tmp)
  (setq expert (getvar "EXPERT")) (setvar "EXPERT" 2)
  (while (not ok)
    (while (not (setq tmp (ustr 0 "Block name" blname nil))))  ;get BLOCK name
    (if (/= tmp blname)                       ;if new block name given
      (progn                                  ;then
        (while
          (not
            (if
              (and (= tmp (no-path tmp))                 ;no path supplied and
                   (setq tbdata (tblsearch "BLOCK" tmp)) ;search table for BLOCK name
              )
              (setq bldata (entget (dxf -2 tbdata)))     ;then get first subentity
              (progn
                (command "INSERT" tmp nil)    ;else insert BLOCK in database, & loop
                (setq tmp (no-path tmp))      ;strip path if any from block name
                nil                           ;return NIL to cause loop
              )
        ) ) );while not if
        (setq attag (dxf 2 bldata))
        (if                    ;if it's an attribute and has the proper format
          (and (= (dxf 0 bldata) "ATTDEF") (= (dxf 2 bldata) "BREAKDIM"))
          (setq bkdist (atof (dxf 1 bldata))  ;then get breakdim data
                blname tmp                    ;and save blname
                ok T                          ;exit flag
          )
          (prompt "\nError: Block must have BREAKDIM attribute as 1st entity.")
        );if an attrib...
      );progn then
      (setq ok T)                             ;else used same block name
    );if new
  );while
  (list blname bkdist)                        ;return block name and break dist
);defun GETBK
;*

;*NO-PATH strips path from a fully qualified file or block name.
;*
(defun no-path (fname / lngth str char)
  (setq lngth (1+ (strlen fname))  str "")  ;initialize, lngth is filename length
  (while                                    ;loops until AND is non-NIL
    (and 
      (/= 1 lngth)                          ;exit to avoid invalid substr  
      (setq lngth (1- lngth)  char (substr fname lngth 1))  ;get next char in reverse
      (/= "/"  char)                        ;find "/" and exit             
      (/= "\\" char)                        ;find "\" and exit                 
      (setq str (strcat char str))          ;otherwise add last char to str 
  ) )
  str
);defun NO-PATH
;*

;* C:BLINE gets a block name, gets its data, draws a line and calls LINEBLK to
;* insert the block and break the line.
(defun C:BLINE ( / linent blkprmt spt ept)    ;bbname and bbdata float for defaults
  (setq bkdata (getbk bbname bbdist)          ;get block name and break data
        bbname (car bkdata)
        bbdist (cadr bkdata)
  )
  (setq spt (getpoint "\nStart point of line: "))     ;get first point of line
  (if spt
    (while (setq ept (getpoint "\nPick end point of line: " spt)) ;get 2nd point of line
      (command "LINE" spt ept "")             ;draw LINE
      (setq linent (entlast))                 ;get insert point
      (setq pt1 (osnap ept "MIDP,QUI"))       ;determine midpoint of LINE 
      (lineblk bbname bbdist pt1 linent)      ;perform Block insert routine
      (setq spt ept)                          ;establish a new start point
      );while
  );if
  (setvar "LASTPOINT" spt)                    ;resets properly to end of line
  (princ)
);defun C:BLINE
;*

;* LINEBLK inserts the autobreaking block and breaks the line or pline
;* To force to 0 degrees or a range -15 to 105 deg, replace the (command "INSERT"...)
;* line with one of the following (the second one requires the ROTANG function, 
;* to force block to read from horizontal or right side). 
; (command "INSERT" blname pt1 #dwgsc "" 0)           ;alt. to force angle to 0 degrees 
; (command "INSERT" blname pt1 #dwgsc "" (angtos (rotang ang nil))) ;restrict angle
;* Or revise programs to use a second attribute in block to control insertion angle.
;
(defun lineblk (blname bkdist pt1 linent / blname brkdim 
                blkname blkent validatt atttag ang)
  (setq pt2 (osnap pt1 "ENDP,QUI"))                   ;get end point for rotation
  (setq ang (angle pt1 pt2))                          ;get angle of line
  (command "INSERT" blname pt1 #dwgsc "" pt2)         ;insert BLOCK 
  (setq pt1 (polar pt1 ang (* (/ bkdist 2) #dwgsc)))  ;calculate 1st break point
  (setq pt2 (polar pt1 (+ ang pi) (* bkdist #dwgsc))) ;calculate 2nd break point
  (command "BREAK" linent pt1 pt2)                    ;break LINE or PLINE
);defun LINEBLK
;*

;* ROTANG converts ANG argument in radians to -15 to 105 degree range, in radians.
;* If align FLAG is True, angles from 105 to 285 are flipped 180 degrees.
;* Note: this is in radians but comments are in degrees for ease of reading.
;*
(defun rotang (ang flag)
  (while (>= ang 6.28318531) (setq ang (- ang 6.28318531)))  ;adj to 0 to 2pi 360 range
  (while (<= ang 0) (setq ang (+ ang 6.28318531)))  
  (cond 
    ((and (>= ang 3.92699081) (<= ang 6.02138591))           ;225-345
      (cond
        ((not flag) (setq ang 0))                            ;to 0
        ((< ang 4.974188368) (setq ang (- ang pi)))          ;225-285 flip 180 deg
        (ang)    
    ) )
    ((and (>= ang 1.83259571) (< ang 3.92699081))            ;105-225
      (if flag
        (setq ang (- ang pi))                                ;flip 180
        (setq ang 1.570796327)                               ;to 90
      ) 
    )
    (ang) 
  )
);defun ROTANG
;*

;* C:BBLOCK uses GETBK and LINEBLK to insert an autobreaking block on existing entities.
(defun C:BBLOCK ( / linent bkdata)            ;bbname and bbdata float for defaults
  (setq linent (entsel "\nPick BLOCK insert point: ")) ;get entity and point
  (if linent
    (progn
      (setq bkdata (getbk bbname bbdist)      ;get the block data
            bbname (car bkdata)               ;store the block name
            bbdist (cadr bkdata)              ;and its break distance
      )
      (setq pt1 (osnap (cadr linent) "NEA"))  ;get nearest point on line
      (lineblk bbname bbdist pt1 (car linent));perform BLOCK insert & break
    );progn
    (prompt "\nMust select Line or Pline. ")
  );if
  (princ)
);defun C:BBLOCK
;*
(princ)
;* end of AUTOBLK.LSP
;*
));test subroutines                                                  
